#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <curl/curl.h>
#include <sstream>
#include <string>


using namespace std;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

struct progress {
  char *pr;
  size_t size;
};

Input input_data(istream& in1, bool prompt) {
    size_t number_count;
    if (prompt) {
    cerr << "Enter number count: ";
    }
    in1 >> number_count;

    Input in;

    in.numbers.resize(number_count);

    for (size_t i = 0; i < number_count; i++)
    {
        in1 >> in.numbers[i];
    }
    if (prompt) {
    cerr << "Enter number bin: ";
    }
    in1 >> in.bin_count;

    return in;
}

//

static int progress_callback(void *clientp,
                      double dltotal,
                      double dlnow,
                      double ultotal,
                      double ulnow)
{


    int progress = (dlnow / dltotal) * 100;
    cerr << " : "<< progress << "%\r";
    //cerr << dlnow << "\n" << dltotal << "\n";

    return 0;
}

//

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);

    return data_size;
}

Input download(const string& address) {
    stringstream buffer;
    progress data;
    CURL* curl = curl_easy_init();
    if(curl)
    {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        //
        curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &data);
        curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, progress_callback);
        curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
        //
        res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if(res != 0)
        {
        cerr << curl_easy_strerror(res);
        exit(1);
        }
    }

    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    curl_global_init(CURL_GLOBAL_ALL);
    Input in;
    if (argc > 1)
    {
        in = download(argv[1]);
    }
    else
    {
        in = input_data(cin, true);
    }

    const auto bins = make_histogram(in.numbers, in.bin_count);
    //show_histogram_text(bins, in.bin_count);
    show_histogram_svg(bins);
    return 0;
}


